scriptName format ["%1Scripts\Server\Functions\Server_BuyUnit.sqf",BIS_WFdPath];
//Last modified 11/17/9
//*****************************************************************************************
//Description: Buys a unit for an AI team.
//*****************************************************************************************
Private["_cost","_data","_empty","_factory","_funds","_index","_position","_side","_soldier","_team","_time","_type","_unitType","_temp_var","_temp_id"];

_type = _this Select 0;
_unitType = _this Select 1;
_factory = _this Select 2;
_empty = _this Select 3;
_team = _this Select 4;
_side = Side _team;

_sideText = WFSideText _side;
if (_side == Civilian) ExitWith {};

_factoryName = Call Compile Format ["%1Structures Select _type",_sideText];
_unit = Call Compile Format["%1%2Units Select _unitType",_sideText,_factoryName];
_cost = Call Compile Format["%1%2Costs Select _unitType",_sideText,_factoryName];
_time = Call Compile Format["%1%2Times Select _unitType",_sideText,_factoryName];

if (time < (BIS_WF_Constants GetVariable "ACCELERATEDBUILDTIME")) then {_time = _time * (BIS_WF_Constants GetVariable "ACCELERATEDBUILDTIMERATIO")};

_funds = _team Call GetTeamFunds;
_index = _team Call BIS_WF_GetAITeamID;
if (_index < 0 || _cost > _funds) ExitWith {_factory SetVariable["inUse",0,true]};

[-_cost,_side,_index] Call ChangeAIFunds; 

_factory SetVariable["inUseType",_unitType,true];
Sleep _time;
_factory SetVariable["inUse",0,true];

if (IsNull _factory) ExitWith {};
if (GetDammage _factory >= 1) ExitWith {};
if (IsPlayer Leader _team) ExitWith {};	//In case a player JIPs into an AI team while constructing units.

_position = [GetPos _factory,38,GetDir _factory + 180] Call GetPositionFrom;

//Query mission config for defined structure types.
_data = [_side,_factoryName] Call BIS_WF_GetBaseStructureData;
if (Count _data > 1) then
{
	_position = [GetPos _factory,_data Select 3,GetDir _factory + (_data Select 4)] Call GetPositionFrom;	
	_position = [[_position,0,20] Call GetRandomPosition,15] Call BIS_WF_GetSafePosition;
};

if (!(_unit Call BIS_WF_IsSoldier)) then
{
	Private["_commander","_crewName","_distance","_driver","_factoryPosition","_gunner","_leader","_pilotName","_soldierName","_templateType","_vehicle"];

	_vehicle = _unit CreateVehicle _position;
	[_vehicle,_side] Call BIS_WF_InitUnit;
	_team AddVehicle _vehicle;

		//add radio action

		if (DIH_RADIO_auto_add_on) THEN
		{
			{
				if (((typeOf _vehicle) isKindOf _x)) THEN
				{
					_temp_var = _vehicle getVariable "DIH_RADIO_present";
					if (isNil ("_temp_var")) THEN {_vehicle setVariable ["DIH_RADIO_present", 0, true];};
					if ((_vehicle getVariable "DIH_RADIO_present") == 0) THEN
					{
						_temp_id = _vehicle addAction ["Use Radio", "DIH_radio\DIH_RADIO_action.sqf",[],-9,false,true,"",""];
						_vehicle setVariable ["DIH_RADIO_present", 1, true];
					};
				};
			} forEach DIH_RADIO_auto_add_white;

			//--- check blacklist

			{
				if (((typeOf _vehicle) isKindOf _x)) THEN
				{
					_temp_var = _vehicle getVariable "DIH_RADIO_present";
					if (isNil ("_temp_var")) exitWith {};
					if ((_vehicle getVariable "DIH_RADIO_present") == 1) THEN
					{
						_vehicle removeAction _temp_id;
						_vehicle setVariable ["DIH_RADIO_present", 0, true];
					};
				};
			} forEach DIH_RADIO_auto_add_black;
		};

	_factoryPosition = GetPos _factory;
	_vehicle SetDir -((((_position Select 1) - (_factoryPosition Select 1)) atan2 ((_position Select 0) - (_factoryPosition Select 0))) - 90);

	//Conform to terrain.
	_vehicle SetVelocity [0,0,-1];

	if (_empty) ExitWith {};

	_soldierName = Call Compile Format["%1SOLDIER",_side Call GetSideLetter];
	_crewName = Call Compile Format["%1CREW",_side Call GetSideLetter];
	_pilotName = Call Compile Format["%1PILOT",_side Call GetSideLetter];

	if (GetText (configFile >> "CfgVehicles" >> _unit >> "crew") == _crewName) then {_soldierName = _crewName};
	if (GetText (configFile >> "CfgVehicles" >> _unit >> "crew") == _pilotName) then {_soldierName = _pilotName};

	//If an Armored team & leader is not in vehicle then move it in.
	_leader = Leader _team;
	_templateType = [_team] Call BIS_WF_GetTeamTemplateTypeName;

	if ((_templateType == Localize "STR_WF_TEAMMECH" || _templateType == Localize "STR_WF_TEAMARM") && _leader == Vehicle _leader /*&& _leader Distance _vehicle < (BIS_WF_Constants GetVariable "BASERANGE")*/) then
	{
		[_leader] AllowGetIn true;
		_leader MoveInDriver _vehicle;
	}
	else
	{
		_driver = _team CreateUnit [_soldierName,_factoryPosition,[],0.2,"FORM"];
		_driver AssignAsDriver _vehicle;
		[_driver] AllowGetIn true;
		[_driver] Join _team;
		_driver MoveInDriver _vehicle;
		[_driver,_side] Call BIS_WF_InitUnit;
	};

	if (_vehicle EmptyPositions "gunner" > 0) then
	{
		_gunner = _team CreateUnit [_soldierName,_factoryPosition,[],0.2,"FORM"];
		[_gunner] AllowGetIn true;
		[_gunner] Join _team;
		_gunner MoveInGunner _vehicle;
		[_gunner,_side] Call BIS_WF_InitUnit;
	};
	
  //Vilem: added commanders to vehicles
  if (_vehicle EmptyPositions "commander" > 0) then
	{
		_commander = _team CreateUnit [_soldierName,_factoryPosition,[],0.2,"FORM"];
		[_commander] AllowGetIn true;
		[_commander] Join _team;
		_commander MoveIncommander _vehicle;
		[_commander,_side] Call BIS_WF_InitUnit;
	};

	//Handle armor team abandoning vehicles.
	if (_templateType == Localize "STR_WF_TEAMARM") then
	{
		_vehicle SetVariable["crew",Crew _vehicle];
		_vehicle AddEventHandler ["GetOut",{_this Spawn BIS_WF_AIVehicleAbandoned}];
	};

	//If leader in near the factory then move vehicle there.
	_distance = Leader _team Distance _vehicle;
	if (_distance > 15 && _distance < (BIS_WF_Constants GetVariable "BASERANGE")) then {_driver DoMove ([Leader _team,6,15] Call GetRandomPosition)};

	if (_vehicle EmptyPositions "cargo" > 0) then {[_vehicle] Spawn BIS_WF_UpdateTransport};
	if (!(BIS_WF_Common GetVariable "coopGame") && !(BIS_WF_Common GetVariable "SPGame")) then {_vehicle Lock true};
}
else
{
	_soldier = _team CreateUnit [_unit,_position,[],0.2,"FORM"];
	[_soldier,_side] Call BIS_WF_InitUnit;
	[_soldier] Join _team;
};

//*****************************************************************************************
//4/13/7 MM - Created file.
