scriptName format ["%1Scripts\Server\AI\Team\Team_UpdateRespawn.sqf",BIS_WFdPath];
//Last modified 12/17/9 - further modified by Dimitri_Harkov
//*****************************************************************************************
//Description: Handles respawn of AI Leader. 
//*****************************************************************************************
Private["_ammo","_barracks","_base","_bodyPosition","_data","_heavy","_heavyTypes","_infantryTypes","_lastBasePosition","_leader","_light","_lightTypes","_list","_position","_side","_sideText","_sorted","_team","_teamTemplate","_weapons"];

_team = _this Select 0;

_side = Side _team;
_sideText = WFSideText _side;

_infantryTypes = Call Compile Format["%1InfantryTeamTypes",_sideText];
_lightTypes = Call Compile Format["%1LightTeamTypes",_sideText];
_heavyTypes = Call Compile Format["%1HeavyTeamTypes",_sideText];

_soldierType = Call Compile Format["%1SOLDIER",_side Call GetSideLetter];

//If default soldier class is not of the same faction as team, then get a new default soldier.
if ((_team GetVariable "faction") != GetText (configFile >> "CfgVehicles" >> _soldierType >> "faction")) then
{
	{
		//Find matching faction in soldiers list.
		if (GetText (configFile >> "CfgVehicles" >> _x >> "faction") == (_team GetVariable "faction")) then
		{
			_soldierType = _x;
		};
	} ForEach Call Compile Format["%1SOLDIERS",_side Call GetSideLetter];
};

_barracks = "";
_light = "";
_heavy = "";

//Query mission config for defined structure types.
_data = [_side,"Barracks"] Call BIS_WF_GetBaseStructureData;
if (Count _data > 0) then {_barracks = _data Select 2};

_data = [_side,"Light"] Call BIS_WF_GetBaseStructureData;
if (Count _data > 0) then {_light = _data Select 2};

_data = [_side,"Heavy"] Call BIS_WF_GetBaseStructureData;
if (Count _data > 0) then {_heavy = _data Select 2};

_weapons = Call Compile Format["%1DefaultWeapons",_sideText];
_ammo = Call Compile Format["%1DefaultAmmo",_sideText];

_lastBasePosition = [];

ScopeName "UpdateRespawn";

while {!gameOver && !(_side In (BIS_WF_Common GetVariable  "defeatedSides"))} do
{
	Sleep 0.5;
	//Remain inactive for player teams and until AI leader is dead.
	while {Alive Leader _team || IsPlayer Leader _team} do {Sleep 0.5};

	_timeToRespawn = time + (BIS_WF_Constants GetVariable "RESPAWNDELAY");
	_bodyPosition = GetPos Leader _team;

	if (_team In playableUnits && IsMultiplayer) then
	{
		//If leader has died then wait for a respawn.
		WaitUntil {Sleep 0.1;Alive Leader _team || IsPlayer Leader _team};
	}
	//If a non-respawnable team leader then create a new leader now.
	else
	{
		Private["_units"];

		_units = Units _team Call GetLiveUnits;
		if (Count _units > 0) then
		{
			_team SelectLeader (_units Select 0);
		}
		else
		{
			WaitUntil {Sleep 0.1; time >= _timeToRespawn};

//			if (!([_side] Call BIS_WF_BaseStructuresExist)) then
			if (_side In (BIS_WF_Common GetVariable  "defeatedSides")) then
			{
//Debug
TextLog Format["WF Debug: %1 non-respawnable team %2 (%3) disbanding since base is destroyed.",_sideText,_team Call BIS_WF_GetTeamID,_team,_leader];

				BreakTo "UpdateRespawn";
			};

			_leader = _team CreateUnit[_soldierType,[Call Compile Format["%1Base",_sideText],10,25] Call GetRandomPosition,[],0.7,"FORM"];
			[_leader,_side] Call BIS_WF_InitUnit;
			_team SelectLeader _leader;

			if (Local player) then
			{
				Private["_commanderTeam"];

				_commanderTeam = Call Compile Format["%1CommanderTeam",_sideText];
				if (!IsNull _commanderTeam) then
				{
					if (_commanderTeam == Group player && player == Leader Group player) then
					{
						player HCSetGroup [_team];
					};
				};
			};
//Debug
TextLog Format["WF Debug: Non-respawnable team %1 (%2) creating new leader (%3).",_team Call BIS_WF_GetTeamID,_team,_leader];
		};
	};
	
	if (!IsPlayer Leader _team) then
	{
		Private["_returnToBase","_units"];

		_units = [];
		_returnToBase = false;
		_leader = Leader _team;

		_base = Call Compile Format["%1Base",_sideText];
		if (!IsNull _base) then {_lastBasePosition = Position _base};	

		//If team was wiped out then respawn at nearest factory.
		if (Count (Units _team Call GetLiveUnits) < 2) then
		{
			Private["_structures"];
			_teamTemplate = _team GetVariable "AITemplate";
			_structures = Call Compile Format["%1BaseStructures",_sideText];
			_list = [];

			//Find closest factory to respawn to.
			if (_teamTemplate In _infantryTypes) then
			{
				{if ((TypeOf _x) == _barracks && GetDammage _x < 1) then {_list = _list + [_x]};} ForEach _structures;
			};

			if (_teamTemplate In _lightTypes) then
			{
				{if ((TypeOf _x) == _light && GetDammage _x < 1) then {_list = _list + [_x]};} ForEach _structures;
			};

			if (_teamTemplate In _heavyTypes) then
			{
				{if ((TypeOf _x) == _heavy && GetDammage _x < 1) then {_list = _list + [_x]};} ForEach _structures;
			};

			_position = [Position _base,10,25] Call GetRandomPosition;
			if (IsNull _base) then {_position = [_lastBasePosition,10,25] Call GetRandomPosition;};

			if (Count _list > 0) then
			{
			_sorted = [_bodyPosition,_list] Call BIS_WF_SortByDistance;
				_position = [Position (_sorted Select 0),10,25] Call GetRandomPosition;
			};

TextLog Format["WF: %1 Team %2 (%3) wiped out.",_sideText,_team Call BIS_WF_GetTeamID, _team];

				//If no team member available, then team has been wiped out.
				//Respawn at the base and run all strategic updates.
				_team SetVariable["nextMediumUpdateTime",time + 30];
				_team SetVariable["nextLongUpdateTime",time - 1];

				//Reset current mission so team can select nearest mission after respawn.
				[_team] Call BIS_WF_RemoveTeamFromMissions;

				//Clear previous destination/regroup at position.
				_leader SetPos _position;
				_team Move _position;
		}
		else
		{
			_templateData = [_team] Call BIS_WF_GetTeamTemplateData;
			if (Count _templateData > 0) then
			{
				//Respawn behavior is based on team type.
				switch (_templateData Select 4) do
				{
					//Try to respawn into passenger in transport first.
					//Then try to respawn in soldier on foot.
					//Then try to respawn into transport driver.
					//If nothing available respawn at base.
					case Localize "STR_WF_TEAMINF":
					{
						Private["_transports"];

						//Get closest functional transport with driver.
						_transports = [_bodyPosition,[Units _team] Call BIS_WF_GetTransports] Call BIS_WF_SortByDistance;

						//Get all living cargo on same team in all transports.
						{
							{
								if (Group _x == _team && _x != Vehicle _x) then {_units = _units + [_x]};
							} ForEach (AssignedCargo _x Call GetLiveUnits);
						} ForEach _transports;

						//If no transport(s), or no passengers in transport(s) then find soldier on foot.
						if (Count _units < 1) then
						{
							_units = [_bodyPosition,Units _team Call GetLiveUnits] Call BIS_WF_SortByDistance;
						};

						//If no one on foot, then get transport driver if available.
						if (Count _units < 1) then
						{
							{
								_returnToBase = true;
								_units = _units + [Driver _x];
							} ForEach _transports;
						};
					};

					//Try to respawn into crew in nearest vehicle first.
					//Then try to respawn in soldier on foot.
					//If nothing available respawn at base.
					case Localize "STR_WF_TEAMMECH":
					{
						Private["_vehicles"];
						ScopeName "Mechanized";

						_vehicles = [_bodyPosition,[Units _team] Call BIS_WF_GetVehicles] Call BIS_WF_SortByDistance;

						//Get closest commander, gunner, or driver.
						{
							if (!IsNull Commander _x) then
							{
								if (Alive Commander _x && Group Commander _x == _team) then
								{
									_units = [Commander _x];
									BreakTo "Mechanized";
								};
							};

							if (!IsNull Gunner _x) then
							{
								if (Alive Gunner _x && Group Gunner _x == _team) then
								{
									_units = [Gunner _x];
									BreakTo "Mechanized";
								};
							};

							if (!IsNull Driver _x) then
							{
								if (Alive Driver _x && Group Driver _x == _team) then
								{
									_units = [Driver _x];
									BreakTo "Mechanized";
								};
							};
						} ForEach _vehicles;

						if (Count _units < 1) then
						{
							_units = [_bodyPosition,Units _team Call GetLiveUnits] Call BIS_WF_SortByDistance;
						};
					};

					case Localize "STR_WF_TEAMARM":
					{
						Private["_vehicles"];
						ScopeName "Armor";

						_vehicles = [_bodyPosition,[Units _team] Call BIS_WF_GetVehicles] Call BIS_WF_SortByDistance;

						//Get closest commander, gunner, or driver.
						{
							if (!IsNull Commander _x) then
							{
								if (Alive Commander _x && Group Commander _x == _team) then
								{
									_units = [Commander _x];
									BreakTo "Armor";
								};
							};

							if (!IsNull Gunner _x) then
							{
								if (Alive Gunner _x && Group Gunner _x == _team) then
								{
									_units = [Gunner _x];
									BreakTo "Armor";
								};
							};

							if (!IsNull Driver _x) then
							{
								if (Alive Driver _x && Group Driver _x == _team) then
								{
									_units = [Driver _x];
									BreakTo "Armor";
								};
							};
						} ForEach _vehicles;
					};
					default
					{
						//Get nearest living team member to death location.
						_units = [_bodyPosition,Units _team Call GetLiveUnits] Call BIS_WF_SortByDistance;
					};
				};
			};

			//Replace available team member with leader.
			if (Count _units > 0) then {[_units Select 0,Leader _team] Spawn BIS_WF_ReplaceTeamMember}
			else
			{
				//If no team member available, then team has been wiped out.
				//Respawn at the base and run all strategic updates.
				_team SetVariable["nextMediumUpdateTime",time + 10];
				_team SetVariable["nextLongUpdateTime",time - 1];
				_leader SetPos ([_lastBasePosition,10,25] Call GetRandomPosition);
				_team Move ([_lastBasePosition,10,25] Call GetRandomPosition);
			};

			if (_returnToBase) then {[_team,[_lastBasePosition,10,25] Call GetRandomPosition] Call BIS_WF_AITeamMove};
	};

	_position = [Position _base,10,25] Call GetRandomPosition;

//Debug
if (IsNil "_position") then
{
	textLogFormat["WF Error: Team_UpdateRespawn null respawn position detected! _base %1 _teamTemplate %2 _list %3",_base,_teamTemplate,_list];

	//Correct respawn.
	_position = [_lastBasePosition,10,25] Call GetRandomPosition;
}
else
{
	if (_position Distance [0,0] < 50) then
	{
		textLogFormat["WF Error: Team_UpdateRespawn [0,0] respawn position detected! _base %1 _teamTemplate %2 _list %3",_base,_teamTemplate,_list];

		//Correct respawn.
		_position = [_lastBasePosition,10,25] Call GetRandomPosition;
	};
textLogFormat["WF Error: Team_UpdateRespawn respawn position %1",_position];
};

			_leader SetPos ([_position,20] Call BIS_WF_GetSafePosition);
			_leader RemoveAllEventHandlers "killed";

			if ((_leader getVariable "DIH_RADIO_present") == 1) THEN
			{
				_leader addEventHandler ["killed", {[_this select 0] spawn DIH_RADIO_RESPAWN}];
			};

		[_leader,_side] Call BIS_WF_InitUnit;
			[_leader,_weapons,_ammo] Call EquipLoadout;						
		};
};

//*****************************************************************************************
//9/2/8 MM - Created file.

