/*
-----------------------------------------------------
--- DIH RADIO OPTIONS SECTION
---
--- see the supplied readme for even more details on the available options.
---
-----------------------------------------------------
*/

/*-------------------
-------------------*/

/*-------------------
friendly group markers:
(string) - what symbols to use for friendly troops "b_" ... BLUFOR, "o_" ... OPFOR, "n_" ... neutral
NOTE: you can use your own set of markers as long as they follow BIS' naming convention.
-------------------*/

DIH_RADIO_marker_NATO_side = "b_";

/*-------------------
enemy group markers:
(string) - what symbols to use for friendly troops "b_" ... BLUFOR, "o_" ... OPFOR, "n_" ... neutral
-------------------*/

DIH_RADIO_marker_NATO_enemy = "o_";

/*-------------------
group marker fade time:
(number) - 1 = 1 hour; when the limit has been reached, markes have faded and group is considered destroyed and will be removed from the marker list.
-------------------*/

DIH_RADIO_marker_expire_time = 0.13;

/*-------------------
transmission time:
(number) - time it takes to complete radio transmission
-------------------*/

DIH_RADIO_transmission_time = 15;

/*-------------------
radio cable length:
(number) - max distance between operator and radio man. Better not change this.
-------------------*/

DIH_RADIO_cable_length = 3;

/*-------------------
mean vehicle radius:
(number) - this distance will be added to the cable length when using vehicle radios from the outside. Necessary because distance is measured from the center of the vehicle.
-------------------*/

DIH_RADIO_veh_diam_const = 4;

/*-------------------
maximum number of radio channels on each side:
(array of numbers) - max number of people that can communicate with the HQ at once (east west guer civ)
-------------------*/

DIH_RADIO_channels_max_ewgc = [6,6,6,6];

/*-------------------
number of currently used radio channels on each side:
(array of numbers) - current number of people that communicate with the HQ (east west guer civ).
Should be 0 for each side at game start - Better not change this.
-------------------*/

DIH_RADIO_channels_cur_ewgc = [0,0,0,0];

/*-------------------
mission factions:
(array of strings and numbers) - all factions that are in the mission need to be listed here, followed by -1
-------------------*/

DIH_RADIO_factions = ["USMC",-1,"INS",-1,"CDF",-1,"GUE",-1,"RU",-1,"CIV",-1,"CIV_RU",-1];

/*-------------------
custom radio transmission speech sounds:
//use the two arrays below to define custom sounds for each faction.
make sure that corresponding entries use the same index pos in each array.
The sounds you use here have to be defined in cfgSounds in description.ext
The DIH Radio default sounds are defined in DIH_configSounds.cpp, that is included your description.ext
-------------------*/

DIH_RADIO_speech_faction_name =		["USMC",		"INS",			"CDF",			"RU",			"GUE",			"CIV",			"CIV_RU"		];
DIH_RADIO_speech_faction_sound =	["DIH_reportinginEN",	"DIH_reportinginRU",	"DIH_reportinginCZ",	"DIH_reportinginRU",	"DIH_reportinginCZ",	"DIH_reportinginCZ",	"DIH_reportinginRU"	];

/*-------------------
Automatically add the 'Use Radio' action to certain units?
//(BOOLEAN) - if set to true, auto adds radios to starting units.
see the following 4 variables for rules
-------------------*/

DIH_RADIO_auto_add_on = true;

/*-------------------
Radio Auto-Add whitelist
//(array of classnames) - whitelist of all vehicle classes that do get radios
this adds radios to all derived classes as well. e.g. "landVehicles" would add radios to all tanks, cars, ...
-------------------*/

DIH_RADIO_auto_add_white = ["tank","air","helicopter"];

/*-------------------
Radio Auto-Add blacklist
//(array of classnames) - blacklist of all vehicle classes that never get radios
 - beware of contradictions, in which case he blacklist has precedence over the whitelist
-------------------*/

DIH_RADIO_auto_add_black = [];

/*-------------------
Radio Auto-Add infantry special condition
//(string) - all infantry units where this condition is true will get a radio,
unless their kind is on the blacklist; _x stands for the unit in question. Default value is "false";

examples:

"leader _x == _x"				--> all squad leaders have radios
"(leader _x == _x) AND (vehicle _x == _x)"	--> all infantry squad leaders have radios
""PortableR168Radio" in weapons _x"	--> everyone with a portable R168 Radio gets the radio ability
(NOTE: This nice piece of equipment does not exist yet)
-------------------*/
		
DIH_RADIO_auto_add_white_cond = "(false)";	

/*-------------------
Standard AI radio usage script on?
//(BOOLEAN) - if set to true, standard AI script for radio operators will be activated.
The standard script ensures that each AI group with radio use it at elast every three minutes.

You may create your own custom scripts for your AI.
Use the following line in your script to let an AI unit use it's radio:

[nil,_operator,"loc",rSPAWN,[_operator],DIH_RADIO_USE] call RE

_operator stands for the person that should use the radio.
-------------------*/

DIH_RADIO_AI_radio_auto_use_standard = true;			

/*-------------------
Info on all friendly starting contacts?
////(BOOLEAN) - if set to true, all friendly groups that are on the map at game start have updated their positions to their commander (i.e: You always see at least all starting positions
of your friendly groups when you use a radio for the first time in a mission)

Does not (yet) work properly in mission where groups get teleported
to other positions right at game start - disable in warfare!
-------------------*/

DIH_RADIO_starting_contacts = true;				

/*-------------------
Show starting contacts?
//(BOOLEAN) - if set to true, all friendly starting contacts will automatically
be marked on each players map on game start.
-------------------*/

DIH_RADIO_auto_mark_starting_contacts = true;				

/*-------------------
Respawn Delay:
//(number) - should be the same as respawnDelay in description.ext
-------------------*/

DIH_RADIO_respawnDelay = 15;			

/*-------------------
Warfare Adjustments:
//(BOOLEAN) - if set to true, some standard warfare Functions will be replaced with adjusted ones that auto-add radios to vehicles that have been bought
an fullfill the white/blacklist requirements. Also, respawned soldiers with radios
will keep their radio ability.

Because this feature might not work with customized warfare modes or because the
adjustments are not needed in none-warfare missions, you can turn this feature off here.
-------------------*/

DIH_RADIO_warfare_adjustments_on = false;				

/*-------------------
Enemy Group Detection Range:
//(number) - enemy contacts up to this distance to operator are reported
 - has serious performance impact, I guess
-------------------*/

DIH_RADIO_enemy_detection_range = 1000;		

/*-------------------
Check for special radio equipment?
//(BOOLEAN) - if set to true, the radio man has to have one of the listed
items below for a transmission
-------------------*/

DIH_RADIO_radio_list_check_toggle = true;

/*-------------------
special radio equipment list:
//(array of item or weapon classname(s)) - the radio man has to have one of the listed items for a transmission, if the above variable is set to true.
- IMPORTANT: lower/upper case DO matter here!
-------------------*/
	
DIH_RADIO_radio_list = ["ItemRadio"];		

/*
Special vehicle vars - use the with 'setVariable' in your scripts:

DIH_RADIO_present		//used by the auto-add radio function and the AI radio usage function. Tells them who has a radio (1 = yes)
DIH_RADIO_inuse			//used by the use_radio.sqf function. do not set manually
DIH_RADIO_external_radio	//set this to 1 for vehicles that have radios that can be accesses from the outside of the vehicle too
*/
