/*
Radio usage script by Dimitri_Harkov

The Plan:
1) check if current radio is in use already, or if operator is authorized to operate it; if not so, abort script - DONE!
2) if not, start talking animation that fits the current stance - !!!TBD: always kneels ATM
3) enable 'abort' action while transmitting; if it's pressed, animation and script will be aborted - DONE!
4) check if a friendly channel is free; if not, abort animation and script - DONE!
5) if so, transmission takes ~20 seconds to complete - DONE!
6) after successful transmission, all friendly groups (but not one's own) are marked on the map together with a time stamp, after all old markers have been deleted - DONE!
7) end animation. - DONE!

---
if _caller != squadleader & not in formation: abort script - DONE!
turn side_channel variable in object associated var? better yes - DONE!

Also: detection & transmission of enemy contacts. - DONE!
The Plan:
1) Check all groups for all enemy groups - DONE!
2) Check if these groups are in the operators detection radius - DONE!
3) If yes, check each group member, the infos available and use a symbol appropriate for the current info. - DONE!
4) Store this info in an array, and send it to the HQ - DONE!
5) Mark these contacts on the map as well, and update the 'knows about' values - DONE!


(!!!TBD: don't transfer friendy marker info: needed only for players -> this info shall be created locally to save ressources

!!!TBD: transfer all larger parts to precompiled functions
!!!TBD: due to use of knowsAbout: call this script where operator is local! - DONE? at least I hope so
*/

private ["_lgm_name", "_lgm_ptr", "_lgm_txt", "_i", "_player_friendly_total", "_player_friendly_groups", "_player_friendly_gr_pos", "_player_friendly_gr_type", "_player_friendly_gr_time", "_player_friendly_gr_wp", "_player_friendly_gr_size", "_cur_list_pos", "_date", "_l_alpha", "_player_friendly_empty", "_operator", "_operator_side", "_operator_g_cs", "_cur_g_m", "_radio_man", "_temp_var", "_side_index", "_eff_cable_length", "_o_anim", "_action_id", "_radio_lang", "_success", "_old_time", "_operator_side_array", "_cur_channels", "_operator_enemy_array", "_local_group_contacts", "_cur_det_units"];

if ((count _this) == 1) THEN
{
	_operator = _this select 0;
	_radio_man = _this select 0;
}
ELSE
{
	_radio_man = _this select 0;
	_operator = _this select 1;
};


_success = 1;


//--- Abort script if the operator doesn't exist. This may seem unneccessary, but it can and has caused problems when the script has been called on extinct AI groups

if (isNull _operator) then {breakTo "quit_script";};


//--- save script success to AI var

if (!(isPlayer _operator)) then {_operator setVariable ["DIH_RADIO_success", 0, false];};



//--- Save the side of the operator. Use playerSide for players, because it's safer than side = ...

if (isPlayer _operator) THEN {_operator_side = playerside} ELSE {_operator_side = side group _operator};

_side_index = switch (_operator_side) do
{
	case east: {0};
	case west: {1};
	case resistance: {2};
	case civilian: {3};
};


//--- pick the right language file for the operator's faction

if ((faction _operator) in DIH_RADIO_speech_faction_name) THEN
{
	_radio_lang = (DIH_RADIO_speech_faction_sound select (([DIH_RADIO_speech_faction_name,(faction _operator)] call BIS_fnc_arrayFindDeep) select 0));
}
ELSE
{
	_radio_lang = "DIH_reportinginEN";
};


//--- Save all living human group members in formation in array. This is done a second time later. That's why its prob better to turn it into a function. - done

_operator_g_cs = [];

_operator_g_cs = [_operator] call DIH_RADIO_fnc_getGroupPlayers;


//------------------------------------------------------------------------------------------------------------------
//--- (1) check if current radio is in use already, or if operator is authorized to operate it; if so, abort script
//------------------------------------------------------------------------------------------------------------------

//--- check for some uninitialized vehicle vars and set them to defaults

_temp_var = _radio_man getVariable "DIH_RADIO_inuse";

if (isnil ("_temp_var")) then {_radio_man setVariable ["DIH_RADIO_inuse", 0, true];};

if (!(_radio_man isKindOf "CAManBase")) THEN
{
	_temp_var = _radio_man getVariable "DIH_RADIO_external_radio";
	if (isnil ("_temp_var")) then {_radio_man setVariable ["DIH_RADIO_external_radio", 0, true];};
};


//--- Exit if Radio is in use
if ((_radio_man getVariable "DIH_RADIO_inuse") == 1) then
{
	if (_operator == player) THEN {hintSilent "Radio currently in use. Try again later.";};
	[_operator_g_cs,_operator,"Patience, The radio is being used already."] call DIH_RADIO_fnc_groupChatToGroup;
	breakTo "quit_script";
};

//--- Exit if Operator is trying to use the radio of an other side
if (side _operator != side _radio_man) then
{
	if (_operator == player) THEN {hintSilent "You do not know how to operate this radio.";};
	[_operator_g_cs,_operator,"I can't. I have no clue how to operate this kind of radio."] call DIH_RADIO_fnc_groupChatToGroup;
	breakTo "quit_script";
};

//--- Exit if Operator is not authorized, i.e neither squad leader nor radio operator nor vehicle commander
if (((leader _operator != _operator) AND (_operator != _radio_man)) AND !((vehicle _operator != _operator) AND (_operator == commander vehicle _operator))) then
{
	if (_operator == player) THEN {hintSilent "You need to be radio operator, vehicle commander, or squad leader to use the radio.";};
	[_operator_g_cs,_operator,"I can't. I'm not authorised to operate a radio."] call DIH_RADIO_fnc_groupChatToGroup;
	breakTo "quit_script";
};

//--- Exit if Operator is trying to use a vehicle radio from the outside. Allow that only for special vehicles (with DIH_RADIO_external_radio set to 1)
if (!(_radio_man isKindOf "CAManBase") AND ((_radio_man getVariable "DIH_RADIO_external_radio") == 0) AND (vehicle _operator == _operator)) then
{
	if (_operator == player) THEN {hintSilent "You need to be inside this vehicle to use its radio.";};
	[_operator_g_cs,_operator,"I can't. I need to get in to use that radio."] call DIH_RADIO_fnc_groupChatToGroup;
	breakTo "quit_script";
};

//--- Exit if Operator is too far away from the squad leader (not necessarily realistic, but good for teamplay)
if ((leader _operator != _operator) AND ((_operator distance (leader _operator)) > 150)) then
{
	if (_operator == player) THEN {hintSilent "You are too far away from the squad leader for an authorized transmission.";};
	[_operator_g_cs,_operator,"I can't hear your instructions. We have to get closer to each other."] call DIH_RADIO_fnc_groupChatToGroup;
	breakTo "quit_script";
};

//--- Exit if operator is infantry and has no radio in his inventory
if ((_radio_man isKindOf "CAmanbase") AND (DIH_RADIO_radio_list_check_toggle)) then
{
	_i = 0;

	{
		if (_x in (weapons _radio_man)) THEN {_i = 1;};
		if (_x in (items _radio_man)) THEN {_i = 1;};
	} forEach DIH_RADIO_radio_list;

	if (_i == 0) THEN
	{
		if (_operator == player) THEN {hintSilent "You do not have a working radio.";};
		[_operator_g_cs,_operator,"I can't. I do not have a working radio."] call DIH_RADIO_fnc_groupChatToGroup;
		breakTo "quit_script";
	};
};

//--- else, mark the radio as beeing used !!don't forget to reset this if script is aborted!!!
_radio_man setVariable ["DIH_RADIO_inuse", 1, true];


//--------------------------------------------------------------------
//--- (2) if not, start talking animation that fits the current stance
//--------------------------------------------------------------------

_o_anim = false;

DIH_RADIO_abort_transmission = false;

if (vehicle _operator == _operator) THEN
{
	_operator playMove "AmovPknlMstpSrasWrflDnon";
	_operator disableAI "ANIM";
	_o_anim = true;

	if (_operator == player) THEN {_action_id = _operator addAction ["Cancel Transmission", "DIH_radio\DIH_cancel.sqf",[],10,true,true,"",""];};

	//add 'abort animation' action to operator - DONE!

};

if (_operator != _radio_man) THEN {_operator doWatch getPos _radio_man;};


//-------------------------------------------------------------------------------
//--- (4) check if a friendly channel is available; if not, abort animation and script
//-------------------------------------------------------------------------------

//--- check if there are available channels

_cur_channels = DIH_Radio_mainscope getVariable "DIH_RADIO_channels_cur";

if ((((DIH_Radio_mainscope getVariable "DIH_RADIO_channels_max") select _side_index) - (_cur_channels select _side_index)) < 1) then
{
	if (_operator == player) THEN {hintsilent "No free radio channels available"; _operator removeAction _action_id;};
	[_operator_g_cs,_operator,"Damn, I can't get through!"] call DIH_RADIO_fnc_groupChatToGroup;
	_radio_man setVariable ["DIH_RADIO_inuse", 0, true];

	if (_o_anim) THEN
	{
		_operator enableAI "ANIM";
		//_operator switchMove "AmovPknlMstpSrasWrflDnon";
	};

	_operator doWatch objNull;
	breakTo "quit_script";
};

//--- add 1 to active channels - deduct, when script over/aborted!

_cur_channels set [_side_index, ((_cur_channels select _side_index) + 1)];
DIH_Radio_mainscope setVariable ["DIH_RADIO_channels_cur", _cur_channels, true];

/* was:
DIH_RADIO_channels_cur_ewgc set [_side_index, ((DIH_RADIO_channels_cur_ewgc select _side_index) + 1)];
publicVariable "DIH_RADIO_channels_cur_ewgc";
*/

//--- save script success to AI var

if (!(isPlayer _operator)) then {_operator setVariable ["DIH_RADIO_success", 1, false];};


//-------------------------------------------------------------------------------
//--- (5) if so, transmission takes ~20 seconds to complete (DIH_RADIO_transmission_time)
//-------------------------------------------------------------------------------

[_operator_g_cs,_operator,"Starting Transmission."] call DIH_RADIO_fnc_groupChatToGroup;
//_operator say [_radio_lang,10];
//was: _operator say3d _radio_lang;
[nil,nil,rEXECVM,"DIH_radio\DIH_RADIO_client_speech.sqf",_operator,_radio_lang] call RE;


if ((_radio_man isKindOf "LandVehicle") OR (_radio_man isKindOf "Helicopter")) THEN
{
	_eff_cable_length = DIH_RADIO_cable_length + DIH_RADIO_veh_diam_const;
}
ELSE
{
	_eff_cable_length = DIH_RADIO_cable_length;
};

_old_time = time;

while {((_old_time + DIH_RADIO_transmission_time) > time) AND !(time < _old_time)} do
{
	sleep 0.1;

	if (_operator == player) THEN {hintSilent "Transmitting...";};

	if (animationState _operator != "AmovPknlMstpSlowWrflDnon_diary") then {_operator playMove "AmovPknlMstpSlowWrflDnon_diary";};

	sleep 0.5;

	if (animationState _operator != "AmovPknlMstpSlowWrflDnon_diary") then {_operator playMove "AmovPknlMstpSlowWrflDnon_diary";};

	sleep 0.4;

	if (((_operator distance _radio_man) > _eff_cable_length) OR (!(alive _operator)) OR (!(alive _radio_man)) OR (!(_o_anim) AND (vehicle _operator == _operator)) OR ((_o_anim) AND (vehicle _operator != _operator)) OR ((isPlayer _operator) AND (DIH_RADIO_abort_transmission))) then 
	{
		if (_o_anim) THEN
		{
			_operator enableAI "ANIM";
		};

		_operator doWatch objNull;
		if (_operator == player) THEN {hintSilent "Transmission aborted."; _operator removeAction _action_id;};
		[_operator_g_cs,_operator,"Transmission aborted."] call DIH_RADIO_fnc_groupChatToGroup;
		_radio_man setVariable ["DIH_RADIO_inuse", 0, true];

		_cur_channels set [_side_index, ((_cur_channels select _side_index) - 1)];
		DIH_Radio_mainscope setVariable ["DIH_RADIO_channels_cur", _cur_channels, true];

		/* was
		DIH_RADIO_channels_cur_ewgc set [_side_index, ((DIH_RADIO_channels_cur_ewgc select _side_index) - 1)];
		publicVariable "DIH_RADIO_channels_cur_ewgc";
		*/

		breakTo "quit_script";
	};
};

if (_o_anim) THEN
{
	_operator enableAI "ANIM";
};

_operator doWatch objNull;
if (_operator == player) THEN {hintSilent "Transmission successful!"; _operator removeAction _action_id;};
[_operator_g_cs,_operator,"Transmission successful!"] call DIH_RADIO_fnc_groupChatToGroup;
_radio_man setVariable ["DIH_RADIO_inuse", 0, true];

_cur_channels set [_side_index, ((_cur_channels select _side_index) - 1)];
DIH_Radio_mainscope setVariable ["DIH_RADIO_channels_cur", _cur_channels, true];

/* was
DIH_RADIO_channels_cur_ewgc set [_side_index, ((DIH_RADIO_channels_cur_ewgc select _side_index) - 1)];
publicVariable "DIH_RADIO_channels_cur_ewgc";
*/


//----------------------------------------------
//--- (6) when transmission has been successful
//----------------------------------------------

//--- Wait until DIH_RADIO_updating is 0 - and then set it to 1

waitUntil {DIH_RADIO_updating == 0};

DIH_RADIO_updating = 1;
publicVariable "DIH_RADIO_updating";


//---------
//get proper Radio Array for local side
//---------

//--- A: friendly side array
_operator_side_array = [];

call compile format["_operator_side_array = DIH_Radio_mainscope getVariable ""DIH_RADIO_fg_reports_%1""", _operator_side];

_player_friendly_total = _operator_side_array select 0;
_player_friendly_groups = _operator_side_array select 1;
_player_friendly_gr_pos = _operator_side_array select 2;
_player_friendly_gr_type = _operator_side_array select 3;
_player_friendly_gr_time = _operator_side_array select 4;
_player_friendly_gr_wp = _operator_side_array select 5;
_player_friendly_gr_size = _operator_side_array select 6;
_player_friendly_empty = _operator_side_array select 7;

//--- B: enemy contacts array
_operator_enemy_array = [];

call compile format["_operator_enemy_array = DIH_Radio_mainscope getVariable ""DIH_RADIO_eg_reports_%1""", _operator_side];

_player_enemy_groups = _operator_enemy_array select 0;
_player_enemy_gr_pos = _operator_enemy_array select 1;
_player_enemy_gr_units = _operator_enemy_array select 2;
_player_enemy_gr_time = _operator_enemy_array select 3;
_player_enemy_empty = _operator_enemy_array select 4;



//-------------------------------
//--- PART A1: get enemy contacts
//-------------------------------

//DIH_RADIO_enemy_detection_range
//_local_group_contacts


_player_enemy_groups_loc = [grpNull];
_player_enemy_gr_pos_loc = [[0, 0, 0]];
_player_enemy_gr_units_loc = [[objNull]];
_player_enemy_gr_time_loc = [0];
_player_enemy_empty_loc = [];


//--- 1) create local group contact array

_i = 1;

{
	if ((side _x != _operator_side) AND (side _x != sideLogic) AND (side _x != civilian)) THEN
	{
		if ((_operator distance getPos leader _x) < DIH_RADIO_enemy_detection_range) THEN
		{
			_cur_det_units = [];

			{
				if (((_operator knowsAbout _x) >= 0.105) AND (alive _x)) THEN
				{
					_cur_det_units = _cur_det_units + [_x];
				};

			} forEach units _x;

			if ((count _cur_det_units) > 0) THEN
			{
				_player_enemy_groups_loc = _player_enemy_groups_loc + [_x];
				_player_enemy_gr_pos_loc = _player_enemy_gr_pos_loc + [getPos leader _x];
				_player_enemy_gr_units_loc = _player_enemy_gr_units_loc + [_cur_det_units];
				_player_enemy_gr_time_loc = _player_enemy_gr_time_loc + [(daytime)];


				//find index of curr group & add all current info to that index
				//_cur_list_pos = ([_player_friendly_groups,group _operator] call BIS_fnc_arrayFindDeep) select 0;

				_i = _i + 1;
			};
		};
	};
} forEach AllGroups;

//if (((count _player_enemy_groups_loc) > 1) AND (isPlayer _operator)) THEN {hint format ["contacts: %1", _player_enemy_groups_loc];};


//--- 2) compare with side contact array and update if necessary

_i = 0;

{

	if (_i != 0) THEN {


	//--- update global data with operator's contacts - case 1: group present

if (_x in _player_enemy_groups) THEN
{
	_cur_list_pos = 0;
	_cur_list_pos = ([_player_enemy_groups,_x] call BIS_fnc_arrayFindDeep) select 0;

	//!!!TBD: For Testing only: only update if new information is more accurate than the old info! - doable?

	//_player_enemy_groups set [_cur_list_pos, (_x)];
	_player_enemy_gr_pos set [_cur_list_pos, (_player_enemy_gr_pos_loc select _i)];
	_player_enemy_gr_units set [_cur_list_pos, (_player_enemy_gr_units_loc select _i)];
	_player_enemy_gr_time set [_cur_list_pos, (_player_enemy_gr_time_loc select _i)];
	
}
ELSE
{
	//--- update global data with operator's contacts - case 2: group needs to be added - no free slots available
	if ((count _player_enemy_empty) < 1) THEN
	{

	_player_enemy_groups = _player_enemy_groups + [_x];
	_player_enemy_gr_pos = _player_enemy_gr_pos + [_player_enemy_gr_pos_loc select _i];
	_player_enemy_gr_units = _player_enemy_gr_units + [_player_enemy_gr_units_loc select _i];
	_player_enemy_gr_time = _player_enemy_gr_time + [_player_enemy_gr_time_loc select _i];

	}
	ELSE
	//--- update global data with operator's contacts - case 3: group needs to be added - free slots are available
	{

	_cur_list_pos = _player_enemy_empty select 0;

	_player_enemy_groups set [_cur_list_pos, (_x)];
	_player_enemy_gr_pos set [_cur_list_pos, (_player_enemy_gr_pos_loc select _i)];
	_player_enemy_gr_units set [_cur_list_pos, (_player_enemy_gr_units_loc select _i)];
	_player_enemy_gr_time set [_cur_list_pos, (_player_enemy_gr_time_loc select _i)];

	_player_enemy_empty = _player_enemy_empty - [(_player_enemy_empty select 0)];

	};
};



};

_i = _i + 1;

} forEach _player_enemy_groups_loc;




//--- 3) update all knowsAbout values of contacts for operator's group

{
	{
		(group _operator) reveal _x;
	} forEach _x;
} forEach _player_enemy_gr_units;


//--- 4) create Markers for player operators and players in formation

// see the standard block below. - DONE!




/*
if ((group _operator) in _player_friendly_groups) THEN
{
	//hint "!";

	_cur_list_pos = ([_player_friendly_groups,group _operator] call BIS_fnc_arrayFindDeep) select 0;

	//hint format ["%1", _player_friendly_gr_pos select _cur_list_pos];

	//sleep 2;

	//_tester = (group player call DIH_RADIO_fnc_getType);

	//hint format ["%1", _tester];

	//sleep 2;

	_player_friendly_gr_pos set [_cur_list_pos, (getPos leader _operator)];
	_player_friendly_gr_type set [_cur_list_pos, (group _operator call DIH_RADIO_fnc_getType)];
	_player_friendly_gr_time set [_cur_list_pos, (daytime)];
	_player_friendly_gr_wp set [_cur_list_pos, (getWPPos [(group _operator),currentWaypoint (group _operator)])];
	_player_friendly_gr_size set [_cur_list_pos, ([group _operator,units group _operator,(group _operator call DIH_RADIO_fnc_getType)] call DIH_RADIO_fnc_getSize)];

	//hint format ["%1", _player_friendly_gr_size select _cur_list_pos];

	//hint format ["%1", _player_friendly_gr_pos select _cur_list_pos];
	
}
ELSE
{
	//--- update data of player's group - case 2: group needs to be added - no free slots available
	if ((count _player_friendly_empty) < 1) THEN
	{

	_player_friendly_groups = _player_friendly_groups	+ [(group _operator)];
	_player_friendly_gr_pos = _player_friendly_gr_pos	+ [(getPos leader _operator)];
	_player_friendly_gr_type = _player_friendly_gr_type	+ [(group _operator call DIH_RADIO_fnc_getType)];
	_player_friendly_gr_time = _player_friendly_gr_time	+ [(daytime)];
	_player_friendly_gr_wp = _player_friendly_gr_wp		+ [(getWPPos [(group _operator),currentWaypoint (group _operator)])];
	_player_friendly_gr_size = _player_friendly_gr_size	+ [([group _operator,units group _operator,(group _operator call DIH_RADIO_fnc_getType)] call DIH_RADIO_fnc_getSize)];

	_player_friendly_total = _player_friendly_total + 1;
	}
	ELSE
	//--- update data of player's group - case 3: group needs to be added - free slots are available
	{

	_cur_list_pos = _player_friendly_empty select 0;

	_player_friendly_groups set [_cur_list_pos, (group _operator)];
	_player_friendly_gr_pos set [_cur_list_pos, (getPos leader _operator)];
	_player_friendly_gr_type set [_cur_list_pos, (group _operator call DIH_RADIO_fnc_getType)];
	_player_friendly_gr_time set [_cur_list_pos, (daytime)];
	_player_friendly_gr_wp set [_cur_list_pos, (getWPPos [(group _operator),currentWaypoint (group _operator)])];
	_player_friendly_gr_size set [_cur_list_pos, ([group _operator,units group _operator,(group _operator call DIH_RADIO_fnc_getType)] call DIH_RADIO_fnc_getSize)];

	_player_friendly_empty = _player_friendly_empty - [(_player_friendly_empty select 0)];

	};
};

*/









//-- (re)set some values for next stage

_date = [];
_operator_g_cs = [];
_cur_list_pos = 0;

//_local_client_markers = [];

_lgm_ptr="";
_lgm_name="";
_lgm_txt="";

//--- PART A - Markers for Players
//------------
//--- marker deletion and placement stuff - player only!
//--- tbd: also run this PART A for all other players in this group, that are in formation with the operator (or: 150m radius - don't  know how to check 'in formation' yet) - DONE

if (isPlayer _operator) THEN
{

	//--- Delete Markers from friendly contacts

	for "_i" from 1 to DIH_RADIO_friend_count do
	{
		_lgm_name = format ["lgm_%1",str(_i)];
		_lgm_name_size = format ["lgm_size_%1",str(_i)];

		//_lgm_ptr = format ["lgm_ptr_%1",str(DIH_RADIO_friend_count)];

		deleteMarkerLocal _lgm_name;
		deleteMarkerLocal _lgm_name_size;

		//hint "!";
	};

	DIH_RADIO_friend_count = 0;


	//--- Delete Markers from enemy contacts

	for "_i" from 1 to DIH_RADIO_enemy_count do
	{
		_lgm_name = format ["lgm_e_%1",str(_i)];
		_lgm_name_size = format ["lgm_size_e_%1",str(_i)];

		deleteMarkerLocal _lgm_name;
		deleteMarkerLocal _lgm_name_size;
	};

	DIH_RADIO_enemy_count = 0;


	//--- Create Markers for friendly contacts

{
//WORKS!	if (!(_operator in units _x)) THEN
//Works better!	if ((group _operator != _x)) THEN
	if (!(isNull _x) and (group _operator != _x)) THEN
	{
		//--- Quit forEach if current operator is HC commander. !!!TBD: what if hcallgroups is not defined?

		if ((count hcallgroups _operator) > 0) exitWith {};

		//--- store current index

		_cur_list_pos = ([_player_friendly_groups,_x] call BIS_fnc_arrayFindDeep) select 0;

		
		//--- Quit IF and store index for deletion if no more group members are left. This should prevent weird situations. This one is indeed not necessary...

		//if (isNull _x) exitWith {_markers_to_delete = _markers_to_delete + [_cur_list_pos]};


		DIH_RADIO_friend_count = DIH_RADIO_friend_count + 1;

		_date = ([_player_friendly_gr_time select _cur_list_pos]) select 0;
		

		//--- group type marker

		_lgm_ptr = format ["lgm_ptr_%1",str(DIH_RADIO_friend_count)];
		_lgm_name = format ["lgm_%1",str(DIH_RADIO_friend_count)];

		_lgm_txt = format ["%1 (%2)", str(_x), [_date,"HH:MM"] call BIS_fnc_timeToString];

		_lgm_ptr = createMarkerLocal[_lgm_name, _player_friendly_gr_pos select _cur_list_pos];
		_lgm_ptr setMarkerShapeLocal "ICON";
		_lgm_name setMarkerTypeLocal (DIH_RADIO_marker_NATO_side + (_player_friendly_gr_type select _cur_list_pos));
		_lgm_name setMarkerColorLocal "ColorBlue";
		_lgm_name setMarkerTextLocal _lgm_txt;



		//--- calculate alpha, store faded markers for deletion.

		_l_alpha = if (((daytime) - (_date)) < 0) THEN
			{(100 - ((24+(daytime - _date))/(DIH_RADIO_marker_expire_time / 100)))/100}
			ELSE
			{(100 - ((daytime - _date)/(DIH_RADIO_marker_expire_time / 100)))/100};



		_lgm_name setMarkerAlphaLocal _l_alpha;


		//--- group size marker

		if ((_player_friendly_gr_size select _cur_list_pos) >= 1) THEN
		{

		_lgm_ptr_size = format ["lgm_ptr_size_%1",str(DIH_RADIO_friend_count)];
		_lgm_name_size = format ["lgm_size_%1",str(DIH_RADIO_friend_count)];
		_lgm_ptr_size = createMarkerLocal[_lgm_name_size, _player_friendly_gr_pos select _cur_list_pos];
		_lgm_ptr_size setMarkerShapeLocal "ICON";
		_lgm_name_size setMarkerTypeLocal ("group_" + str(_player_friendly_gr_size select _cur_list_pos));
		_lgm_name_size setMarkerColorLocal "ColorBlue";

		_lgm_name_size setMarkerAlphaLocal _l_alpha;

		};


		//--- test: instead of deletion, overwrite group name with grpNull and store index of recently created grpNull entry.

		//WORKS! if ((_l_alpha <= 0) and (group player != _x)) THEN
		if ((_l_alpha <= 0) and (true)) THEN
		{
			_player_friendly_empty = _player_friendly_empty + [_cur_list_pos];
			_player_friendly_groups set [_cur_list_pos, grpNull];
		}
		ELSE
		{
		//--- store all groups that have been drawn and not deleted in special array to pass to other human clients in group

			//_local_client_markers = _local_client_markers + [_x];
		};
	};
} forEach _player_friendly_groups;


	//--- Create Markers for enemy contacts

_i = 0;

{

//if (_i == 0) THEN {_i = 1;} ELSE {

//!!!TBD: remove this debug line: if (((count _player_enemy_groups) > 1) AND (isPlayer _operator)) THEN {hint format ["contacts: %1", _player_enemy_gr_units];};

	//was: if (!(isNull (_x))) THEN
	if (!(isNull (_x))) THEN
	{
		//--- Quit forEach if current operator is HC commander. !!!TBD: what if hcallgroups is not defined? !!!TBD: Needed for enemy markers?
		if ((count hcallgroups _operator) > 0) exitWith {};

		//--- store current index

		_cur_list_pos = ([_player_enemy_groups,(_x)] call BIS_fnc_arrayFindDeep) select 0;

		DIH_RADIO_enemy_count = DIH_RADIO_enemy_count + 1;

		_date = ([_player_enemy_gr_time select _cur_list_pos]) select 0;
		

		//--- group type marker

		_lgm_ptr = format ["lgm_ptr_e_%1",str(DIH_RADIO_enemy_count)];
		_lgm_name = format ["lgm_e_%1",str(DIH_RADIO_enemy_count)];

		_lgm_txt = format ["%1", [_date,"HH:MM"] call BIS_fnc_timeToString];

		_lgm_ptr = createMarkerLocal[_lgm_name, _player_enemy_gr_pos select _cur_list_pos];
		_lgm_ptr setMarkerShapeLocal "ICON";

		_lgm_name setMarkerTypeLocal (DIH_RADIO_marker_NATO_enemy +((_player_enemy_gr_units select _cur_list_pos) call DIH_RADIO_fnc_getType_special));

		//_lgm_name setMarkerTypeLocal "o_empty";
		_lgm_name setMarkerColorLocal "ColorRed";
		_lgm_name setMarkerTextLocal _lgm_txt;



		//--- calculate alpha, store faded markers for deletion.

		_l_alpha = if (((daytime) - (_date)) < 0) THEN
			{(100 - ((24+(daytime - _date))/(DIH_RADIO_marker_expire_time / 100)))/100}
			ELSE
			{(100 - ((daytime - _date)/(DIH_RADIO_marker_expire_time / 100)))/100};



		_lgm_name setMarkerAlphaLocal _l_alpha;


		//--- group size marker

		//!!!TBD:? was: if ((_player_friendly_gr_size select _cur_list_pos) >= 1) THEN
		if ((count (_player_enemy_gr_units select _cur_list_pos)) > 0) THEN
		{

		_lgm_ptr_size = format ["lgm_ptr_size_e_%1",str(DIH_RADIO_enemy_count)];
		_lgm_name_size = format ["lgm_size_e_%1",str(DIH_RADIO_enemy_count)];
		_lgm_ptr_size = createMarkerLocal[_lgm_name_size, _player_enemy_gr_pos select _cur_list_pos];
		_lgm_ptr_size setMarkerShapeLocal "ICON";


		_lgm_name_size setMarkerTypeLocal ("group_" + str([(_player_enemy_groups select _cur_list_pos),(_player_enemy_gr_units select _cur_list_pos),((_player_enemy_gr_units select _cur_list_pos) call DIH_RADIO_fnc_getType_special)] call DIH_RADIO_fnc_getSize_special));

		_lgm_name_size setMarkerColorLocal "ColorRed";

		_lgm_name_size setMarkerAlphaLocal _l_alpha;

		};


		//--- test: instead of deletion, overwrite group name with grpNull and store index of recently created grpNull entry.

		//WORKS! if ((_l_alpha <= 0) and (group player != _x)) THEN
		if ((_l_alpha <= 0) and (true)) THEN
		{
			_player_enemy_empty = _player_enemy_empty + [_cur_list_pos];
			_player_enemy_groups set [_cur_list_pos, grpNull];
		}
		ELSE
		{
		//--- store all groups that have been drawn and not deleted in special array to pass to other human clients in group

			//_local_client_markers = _local_client_markers + [_x];
		};
	};

//};
} forEach _player_enemy_groups;


};


//--- PART B - Markers for other human living clients in group
//-----------
//--- 1) create an array of all living human squadmembers of the _operator, that are in range (exclude the operator!)
//--- 2) remote execute DIH_client_markers.sqf for all of them
//
//Do the same for enemy contacts too! - DONE!


_operator_g_cs = [_operator] call DIH_RADIO_fnc_getGroupPlayers;

{
	[nil,_x,"loc",rEXECVM,"DIH_radio\DIH_client_markers.sqf",_operator,_player_friendly_groups,_player_friendly_gr_time,_player_friendly_gr_pos,_player_friendly_gr_type,_player_friendly_gr_size,_player_enemy_groups,_player_enemy_gr_pos,_player_enemy_gr_units,_player_enemy_gr_time] call RE;

} forEach _operator_g_cs;




//--- PART C - update data of operator's group
//-----------
//--- update data of operators group - case 1: group present
//--- !!!TBD: in function auslagern ?

if ((group _operator) in _player_friendly_groups) THEN
{
	//hint "!";

	_cur_list_pos = ([_player_friendly_groups,group _operator] call BIS_fnc_arrayFindDeep) select 0;

	//hint format ["%1", _player_friendly_gr_pos select _cur_list_pos];

	//sleep 2;

	//_tester = (group player call DIH_RADIO_fnc_getType);

	//hint format ["%1", _tester];

	//sleep 2;

	_player_friendly_gr_pos set [_cur_list_pos, (getPos leader _operator)];
	_player_friendly_gr_type set [_cur_list_pos, (group _operator call DIH_RADIO_fnc_getType)];
	_player_friendly_gr_time set [_cur_list_pos, (daytime)];
	_player_friendly_gr_wp set [_cur_list_pos, (getWPPos [(group _operator),currentWaypoint (group _operator)])];
	_player_friendly_gr_size set [_cur_list_pos, ([group _operator,units group _operator,(group _operator call DIH_RADIO_fnc_getType)] call DIH_RADIO_fnc_getSize)];

	//hint format ["%1", _player_friendly_gr_size select _cur_list_pos];

	//hint format ["%1", _player_friendly_gr_pos select _cur_list_pos];
	
}
ELSE
{
	//--- update data of player's group - case 2: group needs to be added - no free slots available
	if ((count _player_friendly_empty) < 1) THEN
	{

	_player_friendly_groups = _player_friendly_groups	+ [(group _operator)];
	_player_friendly_gr_pos = _player_friendly_gr_pos	+ [(getPos leader _operator)];
	_player_friendly_gr_type = _player_friendly_gr_type	+ [(group _operator call DIH_RADIO_fnc_getType)];
	_player_friendly_gr_time = _player_friendly_gr_time	+ [(daytime)];
	_player_friendly_gr_wp = _player_friendly_gr_wp		+ [(getWPPos [(group _operator),currentWaypoint (group _operator)])];
	_player_friendly_gr_size = _player_friendly_gr_size	+ [([group _operator,units group _operator,(group _operator call DIH_RADIO_fnc_getType)] call DIH_RADIO_fnc_getSize)];

	_player_friendly_total = _player_friendly_total + 1;
	}
	ELSE
	//--- update data of player's group - case 3: group needs to be added - free slots are available
	{

	_cur_list_pos = _player_friendly_empty select 0;

	_player_friendly_groups set [_cur_list_pos, (group _operator)];
	_player_friendly_gr_pos set [_cur_list_pos, (getPos leader _operator)];
	_player_friendly_gr_type set [_cur_list_pos, (group _operator call DIH_RADIO_fnc_getType)];
	_player_friendly_gr_time set [_cur_list_pos, (daytime)];
	_player_friendly_gr_wp set [_cur_list_pos, (getWPPos [(group _operator),currentWaypoint (group _operator)])];
	_player_friendly_gr_size set [_cur_list_pos, ([group _operator,units group _operator,(group _operator call DIH_RADIO_fnc_getType)] call DIH_RADIO_fnc_getSize)];

	_player_friendly_empty = _player_friendly_empty - [(_player_friendly_empty select 0)];

	};
};



// --- update global variables when all changes have been made
// -------------------------
// --- !!!tbd: only update values that have changed to save performance! - remove most of the redundant values that could be calculated locally!
// --- Ha, these are the most clumsy lines of code I have ever written... change to faction based 'call compile format [...' stuff later. - DONE!


//--- update friendly contacts

_operator_side_array = [];

_operator_side_array =
	[
		_player_friendly_total,
		_player_friendly_groups,
		_player_friendly_gr_pos,
		_player_friendly_gr_type,
		_player_friendly_gr_time,
		_player_friendly_gr_wp,
		_player_friendly_gr_size,
		_player_friendly_empty
	];

call compile format["DIH_Radio_mainscope setVariable [""DIH_RADIO_fg_reports_%1"", _operator_side_array, true]", _operator_side];


//--- update enemy contacts

_operator_enemy_array = [];

_operator_enemy_array =
	[
		_player_enemy_groups,
		_player_enemy_gr_pos,
		_player_enemy_gr_units,
		_player_enemy_gr_time,
		_player_enemy_empty
	];

call compile format["DIH_Radio_mainscope setVariable [""DIH_RADIO_eg_reports_%1"", _operator_enemy_array, true]", _operator_side];


/* was:
if (_operator_side == east) THEN 	{	DIH_RADIO_fg_reports_total_e = _player_friendly_total;
						DIH_RADIO_fg_reports_names_e = _player_friendly_groups;
						DIH_RADIO_fg_reports_pos_e = _player_friendly_gr_pos;
						DIH_RADIO_fg_reports_type_e = _player_friendly_gr_type;
						DIH_RADIO_fg_reports_time_e = _player_friendly_gr_time;
						DIH_RADIO_fg_reports_waypoint_e = _player_friendly_gr_wp;
						DIH_RADIO_fg_reports_size_e = _player_friendly_gr_size;
						DIH_RADIO_fg_reports_empty_e = _player_friendly_empty;

						publicVariable "DIH_RADIO_fg_reports_total_e";
						publicVariable "DIH_RADIO_fg_reports_names_e";
						publicVariable "DIH_RADIO_fg_reports_pos_e";
						publicVariable "DIH_RADIO_fg_reports_type_e";
						publicVariable "DIH_RADIO_fg_reports_time_e";
						publicVariable "DIH_RADIO_fg_reports_waypoint_e";
						publicVariable "DIH_RADIO_fg_reports_size_e";
						publicVariable "DIH_RADIO_fg_reports_empty_e";	};

if (_operator_side == west) THEN 	{	DIH_RADIO_fg_reports_total_w = _player_friendly_total;
						DIH_RADIO_fg_reports_names_w = _player_friendly_groups;
						DIH_RADIO_fg_reports_pos_w = _player_friendly_gr_pos;
						DIH_RADIO_fg_reports_type_w = _player_friendly_gr_type;
						DIH_RADIO_fg_reports_time_w = _player_friendly_gr_time;
						DIH_RADIO_fg_reports_waypoint_w = _player_friendly_gr_wp;
						DIH_RADIO_fg_reports_size_w = _player_friendly_gr_size;
						DIH_RADIO_fg_reports_empty_w = _player_friendly_empty;

						publicVariable "DIH_RADIO_fg_reports_total_e";
						publicVariable "DIH_RADIO_fg_reports_names_e";
						publicVariable "DIH_RADIO_fg_reports_pos_e";
						publicVariable "DIH_RADIO_fg_reports_type_e";
						publicVariable "DIH_RADIO_fg_reports_time_e";
						publicVariable "DIH_RADIO_fg_reports_waypoint_e";
						publicVariable "DIH_RADIO_fg_reports_size_e";
						publicVariable "DIH_RADIO_fg_reports_empty_e";	};

if (_operator_side == resistance) THEN 	{	DIH_RADIO_fg_reports_total_r = _player_friendly_total;
						DIH_RADIO_fg_reports_names_r = _player_friendly_groups;
						DIH_RADIO_fg_reports_pos_r = _player_friendly_gr_pos;
						DIH_RADIO_fg_reports_type_r = _player_friendly_gr_type;
						DIH_RADIO_fg_reports_time_r = _player_friendly_gr_time;
						DIH_RADIO_fg_reports_waypoint_r = _player_friendly_gr_wp;
						DIH_RADIO_fg_reports_size_r = _player_friendly_gr_size;
						DIH_RADIO_fg_reports_empty_r = _player_friendly_empty;

						publicVariable "DIH_RADIO_fg_reports_total_r";
						publicVariable "DIH_RADIO_fg_reports_names_r";
						publicVariable "DIH_RADIO_fg_reports_pos_r";
						publicVariable "DIH_RADIO_fg_reports_type_r";
						publicVariable "DIH_RADIO_fg_reports_time_r";
						publicVariable "DIH_RADIO_fg_reports_waypoint_r";
						publicVariable "DIH_RADIO_fg_reports_size_r";
						publicVariable "DIH_RADIO_fg_reports_empty_r";	};

if (_operator_side == civilian) THEN 	{	DIH_RADIO_fg_reports_total_c = _player_friendly_total;
						DIH_RADIO_fg_reports_names_c = _player_friendly_groups;
						DIH_RADIO_fg_reports_pos_c = _player_friendly_gr_pos;
						DIH_RADIO_fg_reports_type_c = _player_friendly_gr_type;
						DIH_RADIO_fg_reports_time_c = _player_friendly_gr_time;
						DIH_RADIO_fg_reports_waypoint_c = _player_friendly_gr_wp;
						DIH_RADIO_fg_reports_size_c = _player_friendly_gr_size;
						DIH_RADIO_fg_reports_empty_c = _player_friendly_empty;

						publicVariable "DIH_RADIO_fg_reports_total_c";
						publicVariable "DIH_RADIO_fg_reports_names_c";
						publicVariable "DIH_RADIO_fg_reports_pos_c";
						publicVariable "DIH_RADIO_fg_reports_type_c";
						publicVariable "DIH_RADIO_fg_reports_time_c";
						publicVariable "DIH_RADIO_fg_reports_waypoint_c";
						publicVariable "DIH_RADIO_fg_reports_size_c";
						publicVariable "DIH_RADIO_fg_reports_empty_c";	};
*/


//--- set back global updating var to 0

DIH_RADIO_updating = 0;
publicVariable "DIH_RADIO_updating";

_success = 2;

scopeName "quit_script";

//if (_operator == player) THEN {hint format ["%1", str(_success)];};

_success